import numpy as np
import src.fgh as fgh
import scipy as sp
from src.plot import plot_xy


def V(x: np.array):
    V = 0.5 * (x) ** 2
    return V


if __name__ == "__main__":
    # Input parameters

    A = -10.0
    B = 10.0
    N = 50

    output_indicies = (0, 1, 2, 3, 4)

    # Abscissa x (position)
    x = np.linspace(A, B, N, endpoint=False)

    # Hamiltonian matrix (N x N)
    matH = fgh.fgh_hamiltonian(N, x, V)

    # Diagonalize
    eigenvalues, eigenvectors = sp.linalg.eig(matH, left=False, right=True)

    # Sort eigenvalues and eigenvectors in assending order by real part of the eigenvalues.
    idx = np.argsort(eigenvalues.real)
    eigenvalues = eigenvalues[idx]
    eigenvectors = eigenvectors[:, idx]

    # Normalize to \int |\psi(x)|^2 dx = 1, (Just in case)
    S = np.sum(np.abs(eigenvectors) ** 2, axis=0)
    eigenvectors /= np.sqrt(S)

    # Eigenvector to eigenfunction
    eigenfunctions = eigenvectors * np.sqrt(N / (B - A))

    # Output eigenvalues
    for n in output_indicies:
        print(eigenvalues[n])

    # Output eigenfunctions
    x_dict = {}
    y_dict = {}
    for n in output_indicies:
        x_dict[f"Numeric {n}"] = np.array(x)
        y_dict[f"Numeric {n}"] = eigenfunctions[:, n]
    plot_xy(x_dict, y_dict, show_gui=True, output_image_name="eigenfunctions_on_selection_points.png")

    # # Output eigenfunctions
    # #   Abscissa new x (position)
    # N_plot = 10
    # N_new = 1000
    # x_new = np.linspace(A, B, N_new, endpoint=True)
    # #   Precompute the basis functions as a 2D array
    # basis_new = np.array([fgh.fgh_basis(n, x_new, N, A, B) for n in range(N)], dtype=np.complex128).T
    # #   Compute the eigenfunctions directly using matrix multiplication
    # eigenfunctions_new = basis_new @ eigenvectors[:, :N_plot]
    #
    # x_dict = {}
    # y_dict = {}
    # data_optional = {}
    # for n in output_indicies:
    #     x_dict[f"Numeric {n}"] = np.array(x_new)
    #     y_dict[f"Numeric {n}"] = eigenfunctions_new[:, n].real
    #     data_optional[f"Numeric {n}"] = {"fmt": "-", "lc": f"C{n}"}
    # plot_xy(x_dict, y_dict, data_optional=data_optional, show_gui=True, output_image_name="eigenfunctions_fine.png")
